//==============================================================================
// PROJECT      = HF Generator mit AD9850
// MODULE       = main.C
// SHORT DESC.  = main module
// DEVICE       = 78K0/LG2
// VERSION      = 1.0
// DATE         = 01.09.2005
// LAST CHANGE  =
// =============================================================================


#include "io78f0397.h"
#include "lcd.h"
#include "uart.h"
#include "defines.h"
#include <intrinsics.h>
#include <stdlib.h>

void Wait(unsigned char Number);

//-----------------------------------------------------------------------------
// Option Byte
//-----------------------------------------------------------------------------
#pragma location = "OPTBYTE"
__root const unsigned char opbyte=0x00;

//---------------------------------------------------------------------------------
// Security ID CODE: for OCD On-Chip Debugging
//---------------------------------------------------------------------------------
#pragma location = "SECUID"
__root const unsigned char secuid[11]={0x02,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};


const char alldisplay[] = {255,255,255,255,255,255,255,255,0};
const long steps[8] =
{ 10000000,
   1000000,
    100000,
     10000,
      1000,
       100,
        10,
         1 };


//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
__saddr volatile unsigned char sw3_in;
__saddr long  frequenz;
__saddr long  L;
__saddr int   xposi;
__saddr char  P3merk;
char  display[9];
unsigned long FSWD;
char  obhex;


//-----------------------------------------------------------------------------
// Module name: Wait
// Description: This module delays the program for (number * 50ms).
//-----------------------------------------------------------------------------
void Wait2ms(int number)
{
  TMC50 = 0x0b;	        // initialize Timer50
  TCL50 = 0x07;	        // set input clock to fxp / 2^13 = 976 Hz @ 8MHz => 1ms
//  CR50  = 50;           // set interval time to 50 ms	
  CR50  = 1;           // set interval time to 1 ms	
  TCE50 = 1;		// start Timer50
  while(number > 0)
  { while(!TMIF50);      // wait for Timer50 Interrupt request flag
    TMIF50 = 0;          // clear Timer50 interruupt request flag
    number--;
  }
  TCE50 = 0;		// stop Timer50
}


#define DDClk     1
#define DDFqUd    2
#define DDData    4
#define DDReset   8

void ClkHi (void)
{ P3merk = P3merk | DDClk;
  P3 = P3merk;
}

void ClkLo (void)
{ P3merk = P3merk & ~DDClk;
  P3 = P3merk;
}

void FQUDHi (void)
{ P3merk = P3merk | DDFqUd;
  P3 = P3merk;
}

void FQUDLo (void)
{ P3merk = P3merk & ~DDFqUd;
  P3 = P3merk; }


void DDS_Init (void)
{ P3 = DDReset;        // Reset
  Wait2ms(2);
  P3merk = P3 = 0;
  ClkHi();
  ClkHi();
  ClkLo();
  FQUDHi();
  FQUDLo();
}

/*
N sei das Tuneword.
ddsclock ist 125 MHz.

Frequenz = N *  (ddsclock / 2^32);

also

N = Frequenz * (2^32 / ddsclock);

2^32 / 125 MHz --> 34.35973837
*/

void Calc_N (void)
{ double D;
  D = frequenz;
  FSWD = D  * 34.35973837;
}

void Frequenz_Out (void)
{ int i;
  unsigned long T;

  FQUDLo();
  Calc_N();
  T = FSWD;
  i = 0;
  while (i<32)
  { if (T & 1)
      P3merk |= DDData;
    else
      P3merk &= ~DDData;
    P3 = P3merk;
    ++i;
    ClkHi();
    T = T>>1;
    ClkLo();
  }
  T = 0;
  i = 0;
  while (i<8)
  { if (T & 1)
      P3merk |= DDData;
    else
      P3merk &= ~DDData;
    P3 = P3merk;
    ++i;
    ClkHi();
    T = T>>1;
    ClkLo();
  }
  FQUDHi();
}

extern __saddr char transmit_buffer[5];
extern __saddr char message_byte_count;
extern void  LCD_out(char lcdaddr);

void StelleZeigen ( char stelle, short bitmap)
{
  message_byte_count = 5;
  transmit_buffer [0] = (stelle)<<2;

  transmit_buffer [1] = (bitmap>>12) & 15;
  transmit_buffer [2] = (bitmap>>8) & 15;
  transmit_buffer [3] = (bitmap>>4) & 15;
  transmit_buffer [4] = bitmap & 15;
  LCD_out(LCD_RAM);     // transfer data
}


extern const short characters[43];
#define dezipkt  8
#define topmark  0x1000


void FrequenzToLCD (void)
{ char x;
  char v;
  ldiv_t dt;
  long L;
  short bitmap;

  L = frequenz;
  x = 8;

  if (obhex)
  { L = FSWD;
    while (x)
    { --x;
      v = L & 15;
      if (v>9) v = v + 7;
      bitmap = characters[v];
      if ((x==4)||(x==1)) bitmap |= dezipkt;
      if (xposi==x) bitmap |= topmark;
      StelleZeigen(x, bitmap);
      L = L>>4;
    }
    return;
  }

  while (x)
  { --x;
    dt = ldiv(L,10);
    bitmap = characters[dt.rem];
    if ((x==4)||(x==1)) bitmap |= dezipkt;
    if (xposi==x) bitmap |= topmark;
    StelleZeigen(x, bitmap);
    L = dt.quot;
    if (L==0)
    { while (x)
      { --x;
        StelleZeigen(x, 0);
      }
    }
  }
}



//-----------------------------------------------------------------------------
void main(void)
{
  __disable_interrupt();  // disable all interrupts

  OSCCTL = 0xd0;      // external clock input mode
  PCC = 0x40;         // CPU clock = fxp
  MCM = 0x06;         // Set main system clock and peripheral hardware clock
  OSTS = 0x00;        // Set minimum oscillation stabilization time

  PU7 = 0x1F;         // enable Pull-up resistor
  KRM = 0x1F;         // enable key interrupt detection
  KRMK = 0;           // enable key interrupt

  WTM = 0x83;         // enable watch timer -> interval timer = 0.5 sec
  WTMK = 0;           // enable watch timer interrupt

  PM3 = 0xF0;         // Port 3 = Bit 3..0 = OUT
  __enable_interrupt();   // enable all interrupts

  LCD_init();         // Initialization of LCD controller
  UART_init();        // Initialization of UART6 controller


  LCD_string(alldisplay,0);
  Wait2ms(250);

  sw3_in = 0;
  xposi  = 4;
  obhex  = 0;

  DDS_Init();

  frequenz = 1000000;   // 1 MHz
  Frequenz_Out();
  FrequenzToLCD();


immerzu:
   L = 0;
   if (sw3_in)
   {  switch (sw3_in)
      { case UP:    L = steps[xposi];      break;
        case DOWN:  L = -steps[xposi];     break;
        case RIGHT: if (xposi<7) ++xposi;  break;
        case LEFT:  if (xposi>0) --xposi;  break;
        case SELECT:  obhex = !obhex;      break;
      }
      sw3_in=0;
   }
   if (L)
    { frequenz = frequenz + L;
      if (frequenz < 1) frequenz = 1;
      if (frequenz > 40000000) frequenz =  40000000;
      Frequenz_Out();
      L = 0;
    }
   FrequenzToLCD();
   goto immerzu;
}

//-----------------------------------------------------------------------------
// ISR:      INTKR_vect
// KR0..7 = P7.0 .. P7.7
// knnen mit H->L einen Interrupt auslsen
//-----------------------------------------------------------------------------
#pragma vector = INTKR_vect
__interrupt void isr_INTKR(void)
{ char P7merk, i;
    P7merk = P7;
    i = 255;
    while (i)
    { P7merk = P7merk & P7;
      --i;
    }
    P7merk = ~(P7merk | P7);
    if (!sw3_in)  sw3_in = P7merk & 0x1f;     // read SW3 second time
}
